@@ -1,33 +1,44 @@
-# 
+#
 #  This file is part of Healpy.
-# 
+#
 #  Healpy is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
-# 
+#
 #  Healpy is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU General Public License for more details.
-# 
+#
 #  You should have received a copy of the GNU General Public License
 #  along with Healpy; if not, write to the Free Software
 #  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-# 
+#
 #  For more information about Healpy, see http://code.google.com/p/healpy
-# 
+#
 """Provides input and output functions for Healpix maps, alm, and cl.
 """
+from __future__ import with_statement
+
+import six
+import gzip
+import tempfile
+import shutil
+import os
+import warnings
+
 try:
     import astropy.io.fits as pf
 except ImportError:
     import pyfits as pf
+
 import numpy as np
+
 from . import pixelfunc
 from .sphtfunc import Alm
-import warnings
 from .pixelfunc import UNSEEN
+from . import cookbook as cb
 
 standard_column_names = {
     1 : "I_STOKES",
@@ -38,12 +49,33 @@ standard_column_names = {
 class HealpixFitsWarning(Warning):
     pass
 
+def writeto(tbhdu, filename):
+    # FIXME: Pyfits versions earlier than 3.1.2 had no support or flaky support
+    # for writing to .gz files or GzipFile objects. Drop this code when
+    # we decide to drop support for older versions of Pyfits or if we decide
+    # to support only Astropy.
+    if isinstance(filename, six.string_types) and filename.endswith('.gz'):
+        basefilename, ext = os.path.splitext(filename)
+        with tempfile.NamedTemporaryFile(suffix='.fits') as tmpfile:
+            tbhdu.writeto(tmpfile.name, clobber=True)
+            gzfile = gzip.open(filename, 'wb')
+            try:
+                try:
+                    shutil.copyfileobj(tmpfile, gzfile)
+                finally:
+                    gzfile.close()
+            except:
+                os.unlink(gzfile.name)
+                raise
+    else:
+        tbhdu.writeto(filename, clobber=True)
+
 def read_cl(filename, dtype=np.float64, h=False):
     """Reads Cl from an healpix file, as IDL fits2cl.
 
     Parameters
     ----------
-    filename : str
+    filename : str or HDUList or HDU
       the fits file name
     dtype : data type, optional
       the data type of the returned array
@@ -53,9 +85,8 @@ def read_cl(filename, dtype=np.float64, h=False):
     cl : array
       the cl array
     """
-    hdulist=pf.open(filename)
-    cl = [hdulist[1].data.field(n) for n in range(len(hdulist[1].data.columns))]
-    hdulist.close()
+    fits_hdu = _get_hdu(filename, hdu=1)
+    cl = [fits_hdu.data.field(n) for n in range(len(fits_hdu.columns))]
     if len(cl) == 1:
         return cl[0]
     else:
@@ -82,13 +113,13 @@ def write_cl(filename, cl, dtype=np.float64):
         cols = [pf.Column(name='TEMPERATURE',
                                format='%s'%fitsformat,
                                array=cl)]
-            
+
     tbhdu = pf.new_table(cols)
     # add needed keywords
     tbhdu.header.update('CREATOR','healpy')
-    tbhdu.writeto(filename,clobber=True)
+    writeto(tbhdu, filename)
 
-def write_map(filename,m,nest=False,dtype=np.float32,fits_IDL=True,coord=None,column_names=None):
+def write_map(filename,m,nest=False,dtype=np.float32,fits_IDL=True,coord=None,partial=False,column_names=None,column_units=None,extra_header=()):
     """Writes an healpix map into an healpix file.
 
     Parameters
@@ -97,24 +128,31 @@ def write_map(filename,m,nest=False,dtype=np.float32,fits_IDL=True,coord=None,co
       the fits file name
     m : array or sequence of 3 arrays
       the map to write. Possibly a sequence of 3 maps of same size.
-      They will be considered as I, Q, U maps. 
+      They will be considered as I, Q, U maps.
       Supports masked maps, see the `ma` function.
     nest : bool, optional
       If True, ordering scheme is assumed to be NESTED, otherwise, RING. Default: RING.
       The map ordering is not modified by this function, the input map array
       should already be in the desired ordering (run `ud_grade` beforehand).
     fits_IDL : bool, optional
-      If True, reshapes columns in rows of 1024, otherwise all the data will 
+      If True, reshapes columns in rows of 1024, otherwise all the data will
       go in one column. Default: True
     coord : str
       The coordinate system, typically 'E' for Ecliptic, 'G' for Galactic or 'C' for
       Celestial (equatorial)
+    partial : bool, optional
+      If True, fits file is written as a partial-sky file with explicit indexing.
+      Otherwise, implicit indexing is used.  Default: False.
     column_names : str or list
       Column name or list of column names, if None we use:
       I_STOKES for 1 component,
       I/Q/U_STOKES for 3 components,
       II, IQ, IU, QQ, QU, UU for 6 components,
       COLUMN_0, COLUMN_1... otherwise
+    column_units : str or list
+      Units for each column, or same units for all columns.
+    extra_header : list
+      Extra records to add to FITS header.
     """
     if not hasattr(m, '__len__'):
         raise TypeError('The map must be a sequence')
@@ -130,6 +168,9 @@ def write_map(filename,m,nest=False,dtype=np.float32,fits_IDL=True,coord=None,co
     else:
         assert len(column_names) == len(m), "Length column_names != number of maps"
 
+    if column_units is None or isinstance(column_units, six.string_types):
+        column_units = [column_units] * len(m)
+
     # maps must have same length
     assert len(set(map(len, m))) == 1, "Maps must have same length"
     nside = pixelfunc.npix2nside(len(m[0]))
@@ -138,18 +179,35 @@ def write_map(filename,m,nest=False,dtype=np.float32,fits_IDL=True,coord=None,co
         raise ValueError('Invalid healpix map : wrong number of pixel')
 
     cols=[]
-    for cn, mm in zip(column_names, m):
+    if partial:
+        fits_IDL = False
+        mask = pixelfunc.mask_good(m[0])
+        pix = np.where(mask)[0]
+        if len(pix) == 0:
+            raise ValueError('Invalid healpix map : empty partial map')
+        m = [mm[mask] for mm in m]
+        ff = getformat(np.min_scalar_type(-pix.max()))
+        if ff is None:
+            ff = 'I'
+        cols.append(pf.Column(name='PIXEL',
+                              format=ff,
+                              array=pix,
+                              unit=None))
+
+    for cn, cu, mm in zip(column_names, column_units, m):
         if len(mm) > 1024 and fits_IDL:
             # I need an ndarray, for reshape:
             mm2 = np.asarray(mm)
             cols.append(pf.Column(name=cn,
                                    format='1024%s' % fitsformat,
-                                   array=mm2.reshape(mm2.size/1024,1024)))
+                                   array=mm2.reshape(mm2.size/1024,1024),
+                                   unit=cu))
         else:
             cols.append(pf.Column(name=cn,
                                    format='%s' % fitsformat,
-                                   array=mm))
-            
+                                   array=mm,
+                                   unit=cu))
+
     tbhdu = pf.new_table(cols)
     # add needed keywords
     tbhdu.header.update('PIXTYPE','HEALPIX','HEALPIX pixelisation')
@@ -163,32 +221,58 @@ def write_map(filename,m,nest=False,dtype=np.float32,fits_IDL=True,coord=None,co
     tbhdu.header.update('EXTNAME','xtension',
                         'name of this binary table extension')
     tbhdu.header.update('NSIDE',nside,'Resolution parameter of HEALPIX')
-    tbhdu.header.update('FIRSTPIX', 0, 'First pixel # (0 based)')
-    tbhdu.header.update('LASTPIX',pixelfunc.nside2npix(nside)-1,
-                        'Last pixel # (0 based)')
-    tbhdu.header.update('INDXSCHM','IMPLICIT',
+    if not partial:
+        tbhdu.header.update('FIRSTPIX', 0, 'First pixel # (0 based)')
+        tbhdu.header.update('LASTPIX',pixelfunc.nside2npix(nside)-1,
+                            'Last pixel # (0 based)')
+    tbhdu.header.update('INDXSCHM', 'EXPLICIT' if partial else 'IMPLICIT',
                         'Indexing: IMPLICIT or EXPLICIT')
-    tbhdu.writeto(filename,clobber=True)
+    tbhdu.header.update('OBJECT', 'PARTIAL' if partial else 'FULLSKY',
+                        'Sky coverage, either FULLSKY or PARTIAL')
+
+    # FIXME: In modern versions of Pyfits, header.update() understands a
+    # header as an argument, and headers can be concatenated with the `+'
+    # operator.
+    for args in extra_header:
+        if args[0] == 'COMMENT':
+            tbhdu.header.add_comment(*args[1:])
+        else:
+            tbhdu.header.update(*args)
+
+    writeto(tbhdu, filename)
 
 
-def read_map(filename,field=0,dtype=np.float64,nest=False,hdu=1,h=False,
+def read_map(filename,field=0,dtype=np.float64,nest=False,partial=False,hdu=1,h=False,
              verbose=True,memmap=False):
-    """Read an healpix map from a fits file.
+    """Read an healpix map from a fits file.  Partial-sky files,
+    if properly identified, are expanded to full size and filled with UNSEEN.
 
     Parameters
     ----------
-    filename : str
+    filename : str or HDU or HDUList
       the fits file name
-    field : int or tuple of int, optional
+    field : int or tuple of int, or None, optional
       The column to read. Default: 0.
       By convention 0 is temperature, 1 is Q, 2 is U.
       Field can be a tuple to read multiple columns (0,1,2)
+      If the fits file is a partial-sky file, field=0 corresponds to the
+      first column after the pixel index column.
+      If None, all columns are read in.
     dtype : data type, optional
       Force the conversion to some type. Default: np.float64
     nest : bool, optional
-      If True return the map in NEST ordering, otherwise in RING ordering; 
+      If True return the map in NEST ordering, otherwise in RING ordering;
       use fits keyword ORDERING to decide whether conversion is needed or not
       If None, no conversion is performed.
+    partial : bool, optional
+      If True, fits file is assumed to be a partial-sky file with explicit indexing,
+      if the indexing scheme cannot be determined from the header.
+      If False, implicit indexing is assumed.  Default: False.
+      A partial sky file is one in which OBJECT=PARTIAL and INDXSCHM=EXPLICIT,
+      and the first column is then assumed to contain pixel indices.
+      A full sky file is one in which OBJECT=FULLSKY and INDXSCHM=IMPLICIT.
+      At least one of these keywords must be set for the indexing
+      scheme to be properly identified.
     hdu : int, optional
       the header number to look at (start at 0)
     h : bool, optional
@@ -204,9 +288,10 @@ def read_map(filename,field=0,dtype=np.float64,nest=False,hdu=1,h=False,
     m | (m0, m1, ...) [, header] : array or a tuple of arrays, optionally with header appended
       The map(s) read from the file, and the header if *h* is True.
     """
-    hdulist=pf.open(filename, memmap=memmap)
-    #print hdulist[1].header
-    nside = hdulist[hdu].header.get('NSIDE')
+
+    fits_hdu = _get_hdu(filename, hdu=hdu, memmap=memmap)
+
+    nside = fits_hdu.header.get('NSIDE')
     if nside is None:
         warnings.warn("No NSIDE in the header file : will use length of array", HealpixFitsWarning)
     else:
@@ -215,7 +300,7 @@ def read_map(filename,field=0,dtype=np.float64,nest=False,hdu=1,h=False,
 
     if not pixelfunc.isnsideok(nside):
         raise ValueError('Wrong nside parameter.')
-    ordering = hdulist[hdu].header.get('ORDERING','UNDEF').strip()
+    ordering = fits_hdu.header.get('ORDERING','UNDEF').strip()
     if ordering == 'UNDEF':
         ordering = (nest and 'NESTED' or 'RING')
         warnings.warn("No ORDERING keyword in header file : "
@@ -223,19 +308,64 @@ def read_map(filename,field=0,dtype=np.float64,nest=False,hdu=1,h=False,
     if verbose: print('ORDERING = {0:s} in fits file'.format(ordering))
 
     sz=pixelfunc.nside2npix(nside)
-    if not (hasattr(field, '__len__') or isinstance(field, str)):
-        field = (field,)
     ret = []
 
-    for ff in field:
+    # partial sky: check OBJECT, then INDXSCHM
+    obj = fits_hdu.header.get('OBJECT', 'UNDEF').strip()
+    if obj != 'UNDEF':
+        if obj == 'PARTIAL':
+            partial = True
+        elif obj == 'FULLSKY':
+            partial = False
+
+    schm = fits_hdu.header.get('INDXSCHM', 'UNDEF').strip()
+    if schm != 'UNDEF':
+        if schm == 'EXPLICIT':
+            if obj == 'FULLSKY':
+                raise ValueError('Incompatible INDXSCHM keyword')
+            partial = True
+        elif schm == 'IMPLICIT':
+            if obj == 'PARTIAL':
+                raise ValueError('Incompatible INDXSCHM keyword')
+            partial = False
+
+    if schm == 'UNDEF':
+        schm = (partial and 'EXPLICIT' or 'IMPLICIT')
+        warnings.warn("No INDXSCHM keyword in header file : "
+                       "assume {}".format(schm))
+    if verbose:
+        print('INDXSCHM = {0:s}'.format(schm))
+
+    if field is None:
+        field = range(len(fits_hdu.data.columns) - 1*partial)
+    if not (hasattr(field, '__len__') or isinstance(field, str)):
+        field = (field,)
+
+    if partial:
+        # increment field counters
+        field = tuple(f if isinstance(f, str) else f+1 for f in field)
         try:
-            m=hdulist[hdu].data.field(ff).astype(dtype).ravel()
+            pix = fits_hdu.data.field(0).astype(int).ravel()
         except pf.VerifyError as e:
             print(e)
             print("Trying to fix a badly formatted header")
-            m=hdulist[hdu].verify("fix")
-            m=hdulist[hdu].data.field(ff).astype(dtype).ravel()
-            
+            fits_hdu.verify("fix")
+            pix = fits_hdu.data.field(0).astype(int).ravel()
+
+    for ff in field:
+        try:
+            m=fits_hdu.data.field(ff).astype(dtype).ravel()
+        except pf.VerifyError as e:
+            print(e)
+            print("Trying to fix a badly formatted header")
+            m=fits_hdu.verify("fix")
+            m=fits_hdu.data.field(ff).astype(dtype).ravel()
+
+        if partial:
+            mnew = UNSEEN * np.ones(sz, dtype=dtype)
+            mnew[pix] = m
+            m = mnew
+
         if (not pixelfunc.isnpixok(m.size) or (sz>0 and sz != m.size)) and verbose:
             print('nside={0:d}, sz={1:d}, m.size={2:d}'.format(nside,sz,m.size))
             raise ValueError('Wrong nside parameter.')
@@ -253,24 +383,24 @@ def read_map(filename,field=0,dtype=np.float64,nest=False,hdu=1,h=False,
         except OverflowError:
             pass
         ret.append(m)
-    
+
     if len(ret) == 1:
         if h:
-            return ret[0],hdulist[hdu].header.items()
+            return ret[0],fits_hdu.header.items()
         else:
             return ret[0]
     else:
         if h:
-            ret.append(hdulist[hdu].header.items())
+            ret.append(fits_hdu.header.items())
             return tuple(ret)
         else:
             return tuple(ret)
 
 
 def write_alm(filename,alms,out_dtype=None,lmax=-1,mmax=-1,mmax_in=-1):
-    """Write alms to a fits file. 
-    
-    In the fits file the alms are written 
+    """Write alms to a fits file.
+
+    In the fits file the alms are written
     with explicit index scheme, index = l*l + l + m +1, possibly out of order.
     By default write_alm makes a table with the same precision as the alms.
     If specified, the lmax and mmax parameters truncate the input data to
@@ -280,7 +410,7 @@ def write_alm(filename,alms,out_dtype=None,lmax=-1,mmax=-1,mmax_in=-1):
     ----------
     filename : str
       The filename of the output fits file
-    alms : array, complex
+    alms : array, complex or list of arrays
       A complex ndarray holding the alms, index = m*(2*lmax+1-m)/2+l, see Alm.getidx
     lmax : int, optional
       The maximum l in the output file
@@ -292,7 +422,10 @@ def write_alm(filename,alms,out_dtype=None,lmax=-1,mmax=-1,mmax_in=-1):
       maximum m in the input array
     """
 
-    l2max = Alm.getlmax(len(alms),mmax=mmax_in)
+    if not cb.is_seq_of_seq(alms):
+        alms = [alms]
+
+    l2max = Alm.getlmax(len(alms[0]),mmax=mmax_in)
     if (lmax != -1 and lmax > l2max):
         raise ValueError("Too big lmax in parameter")
     elif lmax == -1:
@@ -307,7 +440,7 @@ def write_alm(filename,alms,out_dtype=None,lmax=-1,mmax=-1,mmax_in=-1):
         mmax = mmax_in
 
     if (out_dtype == None):
-        out_dtype = alms.real.dtype
+        out_dtype = alms[0].real.dtype
 
     l,m = Alm.getlm(lmax)
     idx = np.where((l <= lmax)*(m <= mmax))
@@ -315,35 +448,38 @@ def write_alm(filename,alms,out_dtype=None,lmax=-1,mmax=-1,mmax_in=-1):
     m = m[idx]
 
     idx_in_original = Alm.getidx(l2max, l=l, m=m)
-    
+
     index = l**2 + l + m + 1
 
-    out_data = np.empty(len(index),
-               dtype=[('index','i'),
-                      ('real',out_dtype),
-                      ('imag',out_dtype)])
-    out_data['index'] = index
-    out_data['real'] = alms.real[idx_in_original]
-    out_data['imag'] = alms.imag[idx_in_original]
+    hdulist = pf.HDUList()
+    for alm in alms:
+        out_data = np.empty(len(index),
+                   dtype=[('index','i'),
+                          ('real',out_dtype),
+                          ('imag',out_dtype)])
+        out_data['index'] = index
+        out_data['real'] = alm.real[idx_in_original]
+        out_data['imag'] = alm.imag[idx_in_original]
 
-    cindex = pf.Column(name="index", format=getformat(np.int32), unit="l*l+l+m+1", array=out_data['index'])
-    creal = pf.Column(name="real", format=getformat(out_dtype), unit="unknown", array=out_data['real'])
-    cimag = pf.Column(name="imag", format=getformat(out_dtype), unit="unknown", array=out_data['imag'])
+        cindex = pf.Column(name="index", format=getformat(np.int32), unit="l*l+l+m+1", array=out_data['index'])
+        creal = pf.Column(name="real", format=getformat(out_dtype), unit="unknown", array=out_data['real'])
+        cimag = pf.Column(name="imag", format=getformat(out_dtype), unit="unknown", array=out_data['imag'])
+
+        tbhdu = pf.new_table([cindex,creal,cimag])
+        hdulist.append(tbhdu)
+    writeto(tbhdu, filename)
 
-    tbhdu = pf.new_table([cindex,creal,cimag])
-    tbhdu.writeto(filename,clobber=True)       
-    
 def read_alm(filename,hdu=1,return_mmax=False):
-    """Read alm from a fits file. 
-    
+    """Read alm from a fits file.
+
     In the fits file, the alm are written
     with explicit index scheme, index = l**2+l+m+1, while healpix cxx
-    uses index = m*(2*lmax+1-m)/2+l. The conversion is done in this 
+    uses index = m*(2*lmax+1-m)/2+l. The conversion is done in this
     function.
 
     Parameters
     ----------
-    filename : str
+    filename : str or HDUList or HDU
       The name of the fits file to read
     hdu : int, optional
       The header to read. Start at 0. Default: hdu=1
@@ -373,13 +509,46 @@ def read_alm(filename,hdu=1,return_mmax=False):
 
 ## Generic functions to read and write column of data in fits file
 
-def mrdfits(filename,hdu=1):
-    """Read a table in a fits file.
+def _get_hdu(input_data, hdu=None, memmap=None):
+    """
+    Return an HDU from a FITS file
 
     Parameters
     ----------
-    filename : str
-      The name of the fits file to read
+    input_data : str or HDUList or HDU instance
+        The input FITS file, either as a filename, HDU list, or HDU instance.
+
+    Returns
+    -------
+    fits_hdu : HDU
+        The extracted HDU
+    """
+
+    if isinstance(input_data, six.string_types):
+        hdulist = pf.open(input_data, memmap=memmap)
+        return _get_hdu(hdulist, hdu=hdu)
+
+    if isinstance(input_data, pf.HDUList):
+        if isinstance(hdu, int) and hdu >= len(input_data):
+            raise ValueError('Available hdu in [0-%d]' % len(hdulist))
+        else:
+            fits_hdu = input_data[hdu]
+    elif isinstance(input_data, (pf.PrimaryHDU, pf.ImageHDU, pf.BinTableHDU, pf.TableHDU, pf.GroupsHDU)):
+        fits_hdu = input_data
+    else:
+        raise TypeError("First argument should be a input_data, HDUList instance, or HDU instance")
+
+    return fits_hdu
+
+
+def mrdfits(filename, hdu=1):
+    """
+    Read a table in a fits file.
+
+    Parameters
+    ----------
+    filename : str or HDUList or HDU
+      The name of the fits file to read, or an HDUList or HDU instance.
     hdu : int, optional
       The header to read. Start at 0. Default: hdu=1
 
@@ -388,17 +557,13 @@ def mrdfits(filename,hdu=1):
     cols : a list of arrays
       A list of column data in the given header
     """
-    hdulist=pf.open(filename)
-    if hdu>=len(hdulist):
-        raise ValueError('Available hdu in [0-%d]'%len(hdulist))
-    hdu=hdulist[hdu]
+    fits_hdu = _get_hdu(filename, hdu=hdu)
     val=[]
-    for i in range(len(hdu.columns)):
-        val.append(hdu.data.field(i))
-    hdulist.close()
-    del hdulist
+    for i in range(len(fits_hdu.columns)):
+        val.append(fits_hdu.data.field(i))
     return val
 
+
 def mwrfits(filename,data,hdu=1,colnames=None,keys=None):
     """Write columns to a fits file in a table extension.
 
@@ -422,7 +587,7 @@ def mwrfits(filename,data,hdu=1,colnames=None,keys=None):
     else:
         colnames = ['']*len(data)
     cols=[]
-    for line in xrange(len(data)):
+    for line in six.moves.xrange(len(data)):
         cols.append(pf.Column(name=colnames[line],
                                format=getformat(data[line]),
                                array=data[line]))
@@ -431,7 +596,7 @@ def mwrfits(filename,data,hdu=1,colnames=None,keys=None):
         for k,v in keys.items():
             tbhdu.header.update(k,v)
     # write the file
-    tbhdu.writeto(filename,clobber=True)
+    writeto(tbhdu, filename)
 
 def getformat(t):
     """Get the FITS convention format string of data type t.
