@@ -21,14 +21,16 @@ from . import projector as P
 from . import rotator as R
 from . import pixelfunc
 import matplotlib
-from matplotlib import axes,ticker,colors,cm,lines,cbook,figure
+import matplotlib.axes
 import numpy as np
+import six
+
 from ._healpy_pixel_lib import UNSEEN
 
 pi = np.pi
 dtor = pi/180.
 
-class SphericalProjAxes(axes.Axes):
+class SphericalProjAxes(matplotlib.axes.Axes):
     """Define a special Axes to take care of spherical projection.
 
     Parameters
@@ -255,13 +257,13 @@ class SphericalProjAxes(axes.Axes):
         for xx,yy in zip(x,y):
             if fmt is not None:
                 try: # works in matplotlib 1.3 and earlier
-                    linestyle, marker, color = axes._process_plot_format(fmt)
+                    linestyle, marker, color = matplotlib.axes._process_plot_format(fmt)
                 except: # matplotlib 1.4 and later
-                    linestyle, marker, color = axes._axes._process_plot_format(fmt)
+                    linestyle, marker, color = matplotlib.axes._axes._process_plot_format(fmt)
                 kwds.setdefault('linestyle',linestyle)
                 kwds.setdefault('marker',marker)
                 if color is not None: kwds.setdefault('color',color)
-            l = lines.Line2D(xx,yy,**kwds)
+            l = matplotlib.lines.Line2D(xx,yy,**kwds)
             self.add_line(l)
             thelines.append(l)
         return thelines
@@ -394,7 +396,7 @@ class SphericalProjAxes(axes.Axes):
         elif len(w) >= 2:
             xx.append(x[0:w[0]])
             yy.append(y[0:w[0]])
-            for i in xrange(len(w)-1):
+            for i in six.moves.xrange(len(w)-1):
                 xx.append(x[w[i]:w[i+1]])
                 yy.append(y[w[i]:w[i+1]])
             xx.append(x[w[-1]:])
@@ -729,8 +731,8 @@ def get_color_table(vmin,vmax,val,cmap=None,norm=None):
 def create_colormap(cmap):
     if cmap is not None:
         return cmap 
-    cmap0 = cm.jet
-    newcm = colors.LinearSegmentedColormap('newcm',cmap0._segmentdata,
+    cmap0 = matplotlib.cm.jet
+    newcm = matplotlib.colors.LinearSegmentedColormap('newcm',cmap0._segmentdata,
                                                cmap0.N)
     newcm.set_over(newcm(1.0))
     newcm.set_under('w')
@@ -741,7 +743,7 @@ def create_colormap(cmap):
 #
 #   A Locator that gives the bounds of the interval
 #
-class BoundaryLocator(ticker.Locator):
+class BoundaryLocator(matplotlib.ticker.Locator):
     def __init__(self,N=2):
         if N < 2:
             raise ValueError("Number of locs must be greater than 1")
@@ -772,9 +774,9 @@ class BoundaryLocator(ticker.Locator):
 #   the histogram of data
 #
 
-class HistEqNorm(colors.Normalize):
+class HistEqNorm(matplotlib.colors.Normalize):
     def __init__(self, vmin=None, vmax=None, clip=False):
-        colors.Normalize.__init__(self,vmin,vmax,clip)
+        matplotlib.colors.Normalize.__init__(self,vmin,vmax,clip)
         self.xval = None
         self.yval = None
 
@@ -782,7 +784,7 @@ class HistEqNorm(colors.Normalize):
         if clip is None:
             clip = self.clip
 
-        if cbook.iterable(value):
+        if matplotlib.cbook.iterable(value):
             vtype = 'array'
             val = np.ma.asarray(value).astype(np.float)
         else:
@@ -812,7 +814,7 @@ class HistEqNorm(colors.Normalize):
         if not self.scaled():
             raise ValueError("Not invertible until scaled")
 
-        if cbook.iterable(value):
+        if matplotlib.cbook.iterable(value):
             vtype='array'
             val = np.ma.array(value)
         else:
@@ -847,7 +849,7 @@ class HistEqNorm(colors.Normalize):
         if data.mask is not np.ma.nomask:
             w = w|data.mask
         data2 = data.data[~w]
-        bins = long(min(data2.size/20, 5000))
+        bins = min(data2.size//20, 5000)
         if bins < 3: bins=data2.size
         try:
             # for numpy 1.1, use new bins format (left and right edges)
@@ -898,7 +900,7 @@ class HistEqNorm(colors.Normalize):
 #   A normalization class to get logarithmic color table
 #
 
-class LogNorm2(colors.Normalize):
+class LogNorm2(matplotlib.colors.Normalize):
     """
     Normalize a given value to the 0-1 range on a log scale
     """
@@ -906,7 +908,7 @@ class LogNorm2(colors.Normalize):
         if clip is None:
             clip = self.clip
 
-        if cbook.iterable(value):
+        if matplotlib.cbook.iterable(value):
             vtype = 'array'
             val = np.ma.asarray(value).astype(np.float)
         else:
@@ -942,14 +944,14 @@ class LogNorm2(colors.Normalize):
         ' autoscale only None-valued vmin or vmax'
         if self.vmin is None or self.vmax is None:
             val = np.ma.masked_where(np.isinf(A.data),A)
-            colors.Normalize.autoscale_None(self,val)
+            matplotlib.colors.Normalize.autoscale_None(self,val)
 
     def inverse(self, value):
         if not self.scaled():
             raise ValueError("Not invertible until scaled")
         vmin, vmax = float(self.vmin), float(self.vmax)
 
-        if cbook.iterable(value):
+        if matplotlib.cbook.iterable(value):
             val = np.ma.asarray(value)
             return vmin * np.ma.power((vmax/vmin), val)
         else:
@@ -963,7 +965,7 @@ class LogNorm2(colors.Normalize):
 #   A normalization class to get linear color table
 #
 
-class LinNorm2(colors.Normalize):
+class LinNorm2(matplotlib.colors.Normalize):
     """
     Normalize a given value to the 0-1 range on a lin scale
     """
@@ -971,7 +973,7 @@ class LinNorm2(colors.Normalize):
         if clip is None:
             clip = self.clip
 
-        if cbook.iterable(value):
+        if matplotlib.cbook.iterable(value):
             vtype = 'array'
             val = np.ma.asarray(value).astype(np.float)
         else:
@@ -1006,14 +1008,14 @@ class LinNorm2(colors.Normalize):
         ' autoscale only None-valued vmin or vmax'
         if self.vmin is None or self.vmax is None:
             val = np.ma.masked_where(np.isinf(A.data),A)
-            colors.Normalize.autoscale_None(self,val)
+            matplotlib.colors.Normalize.autoscale_None(self,val)
 
     def inverse(self, value):
         if not self.scaled():
             raise ValueError("Not invertible until scaled")
         vmin, vmax = float(self.vmin), float(self.vmax)
 
-        if cbook.iterable(value):
+        if matplotlib.cbook.iterable(value):
             val = np.ma.asarray(value)
             return vmin + (vmax-vmin) * val
         else:
