@@ -153,12 +153,12 @@ def mollview(map=None,fig=None,rot=None,coord=None,unit='',
         if hasattr(sub,'__len__'):
             nrows, ncols, idx = sub
         else:
-            nrows, ncols, idx = sub/100, (sub%100)/10, (sub%10)
+            nrows, ncols, idx = sub//100, (sub%100)//10, (sub%10)
         if idx < 1 or idx > ncols*nrows:
             raise ValueError('Wrong values for sub: %d, %d, %d'%(nrows,
                                                                  ncols,
                                                                  idx))
-        c,r = (idx-1)%ncols,(idx-1)/ncols
+        c,r = (idx-1)%ncols,(idx-1)//ncols
         if not margins:
             margins = (0.01,0.0,0.0,0.02)
         extent = (c*1./ncols+margins[0], 
@@ -226,7 +226,7 @@ def mollview(map=None,fig=None,rot=None,coord=None,unit='',
         return img
 
 def gnomview(map=None,fig=None,rot=None,coord=None,unit='',
-             xsize=200,ysize=None,reso=1.5,degree=False,
+             xsize=200,ysize=None,reso=1.5,
              title='Gnomonic view',nest=False,remove_dip=False,
              remove_mono=False,gal_cut=0,
              min=None,max=None,flip='astro',
@@ -260,9 +260,7 @@ def gnomview(map=None,fig=None,rot=None,coord=None,unit='',
     ysize : None or int, optional
       The size of the image. Default: None= xsize
     reso : float, optional
-      Resolution (in arcmin if degree is False). Default: 1.5 arcmin
-    degree : bool, optional
-      if True, reso is in degree. Default: False
+      Resolution (in arcmin). Default: 1.5 arcmin
     title : str, optional
       The title of the plot. Default: 'Gnomonic view'
     nest : bool, optional
@@ -322,12 +320,12 @@ def gnomview(map=None,fig=None,rot=None,coord=None,unit='',
         if hasattr(sub,'__len__'):
             nrows, ncols, idx = sub
         else:
-            nrows, ncols, idx = sub/100, (sub%100)/10, (sub%10)
+            nrows, ncols, idx = sub//100, (sub%100)//10, (sub%10)
         if idx < 1 or idx > ncols*nrows:
             raise ValueError('Wrong values for sub: %d, %d, %d'%(nrows,
                                                                  ncols,
                                                                  idx))
-        c,r = (idx-1)%ncols,(idx-1)/ncols
+        c,r = (idx-1)%ncols,(idx-1)//ncols
         if not margins:
             margins = (0.01,0.0,0.0,0.02)
         extent = (c*1./ncols+margins[0], 
@@ -499,12 +497,12 @@ def cartview(map=None,fig=None,rot=None,zat=None,coord=None,unit='',
         if hasattr(sub,'__len__'):
             nrows, ncols, idx = sub
         else:
-            nrows, ncols, idx = sub/100, (sub%100)/10, (sub%10)
+            nrows, ncols, idx = sub//100, (sub%100)//10, (sub%10)
         if idx < 1 or idx > ncols*nrows:
             raise ValueError('Wrong values for sub: %d, %d, %d'%(nrows,
                                                                  ncols,
                                                                  idx))
-        c,r = (idx-1)%ncols,(idx-1)/ncols
+        c,r = (idx-1)%ncols,(idx-1)//ncols
         if not margins:
             margins = (0.01,0.0,0.0,0.02)
         extent = (c*1./ncols+margins[0], 
@@ -670,12 +668,12 @@ def orthview(map=None,fig=None,rot=None,coord=None,unit='',
         if hasattr(sub,'__len__'):
             nrows, ncols, idx = sub
         else:
-            nrows, ncols, idx = sub/100, (sub%100)/10, (sub%10)
+            nrows, ncols, idx = sub//100, (sub%100)//10, (sub%10)
         if idx < 1 or idx > ncols*nrows:
             raise ValueError('Wrong values for sub: %d, %d, %d'%(nrows,
                                                                  ncols,
                                                                  idx))
-        c,r = (idx-1)%ncols,(idx-1)/ncols
+        c,r = (idx-1)%ncols,(idx-1)//ncols
         if not margins:
             margins = (0.01,0.0,0.0,0.02)
         extent = (c*1./ncols+margins[0],
